# by DCL manual
#english
=begin
==NumRu::AdvancedDCL::Contour
===About this module
The Contour is a module for drawing 2-D contour graphs. The contour lines are drawn in the UC, and are compatible with all normalization transformations. This package only draws the contour lines, and does not draw the outer frames 
such as the coordinate axes. 

 The following are the subroutines for the user in the Contour. 
* Draw methods
  * draw DDDDraws a 2-D contour graph
* Methods for contour level
  * gcla DDDGenerates contour by minimum & maximum values and step size.
  * gclb DDDGenerates contour by the array giving the grip point values and step size.
  * iclv DDDInitializes (clears) a contour level.
  * sclv DDDGenerates a single contour level.
  * dclv DDDDeletes a single contour level.
* ̑̃\bh
  * set, get DDDInquires/changes an internal variable.

If imformation concerning grid-point coordinates are given a drawing method, grid-point coordinates are set automatically.

If you wish to have a quick look at the contour for a 2-D data set, then simply make a call to Contour.draw. Evenly-spaced grid points will be set to the full extent of the window, and the contour level will be generated automatically.

If the grid points are not evenly spaced, then give Contour.draw method the coordinate values of the grid points . 

Or, when you wish to specify the contour level beforehand, then call Contour.gcla or Contour.gclb before making the call to Contour.draw. However, in this case, only contour levels with equal intervals are generated. If you wish to generate contour levels at uneven intervals, or if you wish to add a specific contour level, generate each contour level using Contour.sclv. (To clear a single contour level, call Contour.dclv. To clear all contour levels, call Contour.iclv.)

===Parameters
Name  Type     DCL package
:indexmj (Fixnum) <UDPACK>
   >The line index of the major line. (The initial value is 3.)
:indexmn (Fixnum) <UDPACK>
   The line index of the minor line. (The initial value is 1.)
:label (true or false) <UDPACK>
   Specifies whether to place labels on major lines. Labeled when true not labeled when false. (The initial value is true)
:ldash (true or false) <UDPACK>
   Specifies whether to use different line types for drawing negative contour lines. When true, the negative contour lines are drawn with line type inquired by the internal variable 'idash'; when false, all contour lines are drawn with the line type inquired by the internal variable 'isolid'. (The initial value is true).
:idash (Fixnum) <UDPACK>
   The line type used for the negative contour level when 'ldash' is true. (The initial value is 3: dashed line.)
:isolid (Fixnum) <UDPACK>
   The line type used for contour levels with values larger than 0 when 'ldash'
is true, and for all contour levels when 'ldash' is false. (The initial value is 1: solid line.)
:icycle (Fixnum) <UDPACK>
   Specifies the number of minor lines to drawn between major lines. One major line is drawn for a cycle of contour lines specified by the value inquired by 'icycle', and the remaining contour lines are minor lines. (The initial value is 2.)
:nlev (Fixnum) <UDPACK>
   When contour levels are automatically generated, this internal variable is inquired to generate nlev contour levels. (The initial value is 12.)
:labelsize (Float) <UDPACK>
   The character height of the contour label. (Units in VC; The initial value is undefined.) When this internal variable is undefined, the internal variable 'rsizel1' handled by Axis::Lin is inquired.
:lmsg (true or false) <UDPACK>
   Specifies whether to write a message in the bottom margin. Message printed when true, not printed when false. (The initial value is true)
:titlesize (Float) <UDPACK>
   The character height of the message printed in the bottom margin of the graph. (Units in VC; The initial value is undefined.) When this internal variable is undefined, the internal variable 'rsizel1' handled by Axis::Lin is inquired.
:xttl (Float) <UDPACK>
   An internal variable which determines the position of the message printed in the bottom margin of the graph. Takes a floating-point value between -1 to +1. Left-aligned when -1, centered when 0, and right-aligned when +1. (The initial value is 0.).
:labelfmt (String) <UDPACK::UDSFMT>
   Specifies the format of the contour label. For example, when you wish to make all contour labels integer types, then    
     Contour.labelfmt = '(I8)'.
   (Since the characters are left aligned, the result is the same for example, with '(I12)', if the label is shorter than 8 characters. The initial value is the option 'D' (an expression 4-to-5-characters long when the number of significant digits is 3), interpreted by the subroutine <tt>CHVAL</tt> (see section on FMTLIB in the user's manual for "MISC1").  
:iundef (Fixnum) <SYSLIB>
   Integer-type value indicating 'not specified'. (Initial value: -999)
:rundef (Float) <SYSLIB>
   Floating-point-number-type value indicating 'not specified'. (Initial value: -999.0)
:lmiss (true or false) <SYSLIB>
   Specifies whether missing-value handling will be performed. Performed when true, and not performed when false (Initial value: false).
:imiss (Fixnum) <SYSLIB>
   Specifies the integer-type missing value for missing-value handling. (Initial value: 999)
:rmiss (Float) <SYSLIB>
   Specifies the floating-point-number-type missing value for missing-value handling. (Initial value: 999.0)

In addition, These parameters can be used as methods. For example, it is possible to refer the parameter "xttl" with Contour.xttl method. Moreover, it is possible to set the parameter "xttl" with Contour.xttl= method.

=end
#japanese
=begin
==NumRu::AdvancedDCL::Contour
==={W[ɂ
ContourW[͓}`߂̃W[łB
UWnō悳̂, ׂĂ̐KϊɑΉĂ. ̃W[ł, }`ōWȂǂ̊Og͕`Ȃ. 

Contour ɂă[U[gp郁\bhƂĂ͈ȉ̂悤Ȃ̂. 
* 惁\bh
  * draw DDD2}悷 
* R^[xɊւ郁\bh 
  * gcla DDDŏlEőlEݒlŐ.
  * gclb DDDiq_l^zƂݒlŐ.
  * iclv DDDR^[x𖳌ɂ.
  * sclv DDD1{̃R^[x𐶐.
  * dclv DDD1{̃R^[x폜.
* ̑̃\bh
  * set, get DDDϐQ/ύX.

iq_WɊւẮA惁\bhɊiq_(WlƂ)^ƎIɐݒ肵Ă.

2f[^ƂɂR^O(quick look)ƂƂɂ, P Contour.draw ĂԂł悢. ̂Ƃ, ݐݒ肳ĂEChEς Ԋůiq_ݒ肵, R^[xIɐ. 

iq_sԊȕꍇ, iq_̍Wl`惁\bḧƂĎw肷΂悢.

܂R^[x炩ߎw肵Ƃ, Contour.draw ĂԑO Contour.gcla ܂ Contour.gclb Ăׂ΂悢. ̏ꍇ, ԊũR^[xȂ. , sԊũR^[xw肵ꍇ, ̃R^[xǉꍇ, Contour.sclv 1{1{̃R^[x𐶐Ă΂悢. (1{̃R^[x𖳌ɂɂ Contour.dclv Ăׂ΂悢; ׂẴR^[x𖳌ɂɂ Contour.iclv Ăׂ΂悢. ) 

===p^
p^ p^̌^ ΉDCLpbP[W
:indexmj (Fixnum) <UDPACK>
   major line ̃CCfNX(l3).
:indexmn (Fixnum) <UDPACK>
   minor line ̃CCfNX(l1).
:label (true or false) <UDPACK>
   major line Ƀx邩ǂ̎w. true Ȃ; false ȂȂ(l true ).
:ldash (true or false) <UDPACK>
   ̃R^[CقȂ郉C^Cvŕ`ǂw肷. true ̂Ƃ̃R^[Cp^ idash ̎QƂ郉C^Cvŕ`; false ̂ƂׂẴR^[Cp^ isolid ̎QƂ郉C^Cvŕ`(l true ).
:idash (Fixnum) <UDPACK>
   ldash  true ̂Ƃ,̃R^[xɎgC^Cv(l3:j).
:isolid (Fixnum) <UDPACK>
   ldash  true ̂Ƃ0ȏ̃R^[x,܂ ldash  false ̂ƂׂẴR^[xɎgC^Cv(l1:).
:icycle (Fixnum) <UDPACK>
   minor line  major line ̊Ԃɉ{w肷. Ȃ킿 icycle ŎQƂl1{̊ major line , c minor line (l2).
:nlev (Fixnum) <UDPACK>
   R^[xIɐƂ,̓ϐQƂĖ nlev {̃R^[x𐶐(l12).
:labelsize (Float) <UDPACK>
   R^[x̕(PʂVWn;l͕s).̓ϐŝƂ Axis ̊Ǘp^ rsizel1QƂ( Axis Q).
:lmsg (true or false) <UDPACK>
   Ot̉}[WɃbZ[Wǂw肷. true Ȃ珑; false Ȃ珑Ȃ(l true ).
:titlesize (Float) <UDPACK>
   Ot̉}[WɏbZ[W̕(PʂVWn;l͕s).̓ϐŝƂ Axis ̊Ǘp^ rsizel1 QƂ(3.7ߎQ).
:xttl (Float) <UDPACK>
   Ot̉}[WɏbZ[Ẅʒu߂ϐ. -1  +1 ܂ł̎lƂ肤. -1 ̂Ƃ悹, 0 ̂ƂZ^O, +1 ̂ƂE悹(l 0 ).
:labelfmt (String) <UDPACK::UDSFMT>
   R^[x̃tH[}bgw肷(16ȉ). Ƃ΃R^[xׂĐ^ɂƂ 
     Contour.labelfmt = '(I8)' 
   Ƃ΂悢 (͍l߂̂Ńx8ȉȂ, Ƃ '(I12)' łʂ͓). lDCL(rbdcl)̃Tu[`CHVAL (uMISC1ṽ}jAFMTLIBQ)߂IvV'D' (L3P^łق4-5̕\)ł. 
:iundef (Fixnum) <SYSLIB>
   sł邱Ƃ^̒l(l-999)
:rundef (float) <SYSLIB>
   sł邱Ƃ^̒l(l-999.0)
:lmiss (true or false) <SYSLIB>
   lȂ ǂw肷. true Ȃ炨Ȃ, false Ȃ炨ȂȂ (l false).
:imiss (Fixnum) <SYSLIB>
   lȂƂ̐^̌lw肷 (l999).
:rmiss (Float) <SYSLIB>
   lȂƂ̎^̌lw肷 (l999.0).

܂Ãp^̓\bhƂėp邱ƂłBȂ킿AContour.xttl Ńp^ xttl ̎QƂAContour.xttl= Ńp^ xttl ̕ύXłB

=end

module NumRu
  module AdvancedDCL
    module Contour
      def param_init
	@param = Parameter.new(Common.param)
	@param.init("indexmj", Fixnum, "indexmj", "indexmjget", 3, 2)
	@param.init("indexmn", Fixnum, "indexmn", "indexmnget", 3, 2)
	@param.init("label", TrueClass, "udlset", "udlget", 0, 0)
	@param.init("ldash", TrueClass, "udlset", "udlget", 0, 0)
	@param.init("idash", Fixnum, "udiset", "udiget", 0, 0)
	@param.init("isolid", Fixnum, "udiset", "udiget", 0, 0)
	@param.init("icycle", Fixnum, "udiset", "udiget", 0, 0)
	@param.init("nlev", Fixnum, "udiset", "udiget", 0, 0)
	@param.init("labelsize", Float, "labelsize", "labelsizeget", 3, 2)
	@param.init("lmsg", TrueClass, "udlset", "udlget", 0, 0)
	@param.init("titlesize", Float, "titlesize", "titlesizeget", 3, 2)
	@param.init("xttl", Float, "udrset", "udrget", 0, 0)
	@param.init("iundef", Fixnum, "udiset", "udiget", 0, 0)
	@param.init("labelfmt", String, "udsfmt", "udqfmt", 1, 1)

	@param.init("lmiss", TrueClass, "gllset", "gllget", 0, 0)
	@param.init("imiss", Fixnum, "gliset", "gliget", 0, 0)
	@param.init("rmiss", Float, "glrset", "glrget", 0, 0)
	@param.init("rundef", Float, "glrset", "glrget", 0, 0)
	
	@temp_param = @param.dup
	@backup = @param.dup
	@orig_param = @param.dup
	@set = Hash.new
      end
      
      private :param_init

#------------------- method for parameters----------------------

      include Parameter_appli
      
# by kawanabe
#english
=begin
---Contour.set(key, val)
   Sets parameters. If val is nil, the value of the parameter will return default value in DCL.

   Arguments
   * key: parameter name(String)
   * val: new value. About type, refer each parameter.

   Example
      Box.set("xttl", 3)      # the position of the message Left-aligned

=end
#japanese
=begin
---Contour.set(key, val)
    p^̐ݒsBvalnilw肷ƒlDCLɂ鏉lɖ߂B

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB
    gp
        Contour.set("xttl", -1)      # bZ[W񂹂ɂB

=end

# by kawanabe
#english
=begin
---Contour.get(key)
    Refers the value of the parameter.

    Arguments
    * key: parameter name(String)

    Example
        Contour.get("xttl")      # Sees the position of the message

=end
=begin
---Contour.get(key)
    p^QƂB

    
    * key:p^(String)
    gp
        Contour.get("xttl")      # bZ[ẄʒuQƂB

=end

# by kawanabe
#english
=begin
---Contour.reset(key)
    Sets the value of the parameter default value.

    Arguments
    * key: parameter name(String)

    Example
        Box.reset("xttl")      # Sets the position of the message a default position.

=end
#japanese
=begin
---Contour.reset(key, val)
    p^DCLɂ鏉lɖ߂B

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB

    gp
        Contour.reset("xttl")      # bZ[Ẅʒulɖ߂

=end

#english
=begin
---Contour.help_params
    Refers available parameters in Contour module.

    Arguments
    None

    Notes
    (1)This method return name of the parameter, present value, type of the parameter.

=end
#japanese
=begin
---Contour.help_params
    ContourŐݒ\ȃp^QƂB

    
    Ȃ

    l
    (1)̃\bh͐ݒ\ȃp^ɊւāAp^A݂̒lAp^̌^ԂB

=end

#english
=begin
---Contour.sync
    Synchronizes the value of a parameter in DCL with the present value of the parameter in AdvancedDCL.

    Arguments
     None

    Notes
     In AdvancedDCL, parameters which set with "set" method do not synchronize with DCL and a method to synchronize parameters is called in drawing methods. This method is used to synchronize parameters with these of DCL directly.

=end 
#japanese
=begin
---Contour.sync
    ݁Aݒ肳Ăp^DCLɔfB

    
     Ȃ

    l
     AdvancedDCLɂĂset\bĥ݂ł̓p^DCLɔfꂸA
p^sync郁\bhŌĂłB̃\bh̓[Up
^zsyncł邽߂̃\bhłB

=end
 
      def gcla(max, min, dx)
	@param.sync
	
	DCL.udgcla(max, min, dx)
	nil
      end

# by DCL manual
#english
=begin
---Contour.gcla(max, min, dx)
    Sets the value of the contour level used by Contour.draw by the minimum & maximum values and step size, or by the number of levels.

    Arguments
    * max, min (Float) : The minimum and maximum values of the contour levels. The levels do not necessarily need to have conveniently rounded numbers.
    * dx (Float) : When DX &lt 0, uses DX as the step size. When DX=0, inquires 'NLEV' and generates approximately  NLEV contour levels. When DX &gt 0, generates approximately ABS(DX) contour levels.

    Notes
    (1) UDGCLA is called in this method.
    (2) Must be XMIN != XMAX. 
    (3) The contour level is set as the step size specified by DX multiplied by an integer factor.
    (4) The labels for the contours are generated by the subroutine UDLABL. The format used in this case can be inquired/changed using the parameter 'labelfmt'
    (5) There are 2 types of contours; the major line and the minor line. The major line is a labeled contour, and the minor line is not labeled. (However, by setting the internal variable 'label' to false, the major lines will also have no labels. The initial value is true) The line index of the major line is specified by the internal variable 'indexmj'. (The initial value is 3.) The character height for the label of the major line is specified by the internal variable 'rsizel'. (Units in values of VC; the initial value is undefined. Here, RSIZEL inquires the internal variable 'rsizel1' handled by Axis::Lin.) The line index of the minor line is specified by the internal variable 'indexmn'. (The initial value is 1.)
    (6) When the internal variable 'ldash' is true, the contour lines with negative contour levels are drawn using the line type of the inquired internal variable 'idash'. (The initial value of 'ldash' is true; the initial value of 'idash' is 3: dashed lines.) All other contour lines are drawn with line types of the inquired internal variable 'isolid'. (The initial value of 'isolid' is 1: solid line.)
    (7) The number of minor line between two major lines is specified by the internal variable 'icycle'. (The initial value is 2.) In other words, one major line is drawn for a cycle of contours specified by the inquired value of 'icycle', and the remaining contours in the cycle are minor lines. The contour levels of the major line is set as the step size specified by DX multiplied by the factor 'icycle'.

=end
#japanese
=begin
---Contour.gcla(max, min, dx)
    Contour.draw ŎgR^[xl ŏlEől ݒl܂͂悻̃xŐݒ肷.

    
    * max, min (Float) : R^[x̍ŏlƍől. K̂悢lłȂĂ悢.
    * dx (Float) : dx0̂Ƃ dxݕƂ. dx=0̂Ƃp^ nlev QƂ,  nlev { R^[x𐶐. dx0̂Ƃ (dx.abs.to_i) {̃R^[x𐶐. 

    l
    (1) ̃\bhUDGCLAĂł.
    (2) xminxmaxłȂ΂ȂȂ. 
    (3) R^[x dx ߂邫ݕ̐{ɂȂ悤ɐݒ肳. 
    (4) R^[ɂ郉x, DCL(rbdcl)Tu[`UDLABL. ̂ƂgptH[}bg, p^ labelfmt ɂQ/ύXł. 
    (5) R^[ɂ major line  minor line 2ނ. major line Ƃ̓xt̃R^[, minor line Ƃ̓xȂ̃R^[ł. (, p^ label  false ɂ major line xȂƂ邱Ƃł. l true ) major line ̃CCfNX̓p^ indexmj ߂(l3). major line ɕt郉x̓̕p^ rsizel ߂(PʂVWn̒l;l͕s. ̂Ƃ rsizel  Axis ̊Ǘϐ rsizel1 QƂ). minor line ̃CCfNX͓ϐ indexmn ߂(l1). 
    (6) p^ ldash  true ̂Ƃ, R^[x̃R^[C͓ϐ idash QƂẴC^Cvŕ` ( ldash ̏l true, idash ̏l3:j). ȊÕR^[C̓p^ isolid QƂẴC^Cvŕ` ( isolid ̏l1:). 
    (7) minor line  major line ̊Ԃɉ{͓ϐ icycle (l2)߂. Ȃ킿 icycle ŎQƂl1{̊ major line , c minor line . major line ̃R^[x dx Ō܂邫ݕ icycle {ɂȂ悤ɐݒ肳.

=end

      def gclb(z, dx)
	@param.sync
	
	DCL.udgclb(z, z.shape[0], z.shape[0], z.shape[1], dx)
	nil
      end

# by DCL manual
#english
=begin
---Contour.gclb(z, dx)
    Sets the value of the contour level used by Contour.draw with an array giving the grid-point values and the step size or the approximate number of levels. 
    Arguments
    * z (NArray) : 2-D Array 
    * dx (Float) : When DX &lt 0, uses DX as the step size. When DX=0, inquires 'NLEV' and generates approximately  NLEV contour levels. When DX &gt 0, generates approximately ABS(DX) contour levels.

    Notes
    (1) UDGCLB is called in this method.
    (2) This routine checks the minimum and maximum values of the grid-point values for array Z, and calls Contour.gcla internally.

=end
#japanese
=begin
---Contour.gclb(z, dx)
    Contour.draw ŎgR^[xliq_l^zƂݒl܂͂悻̃xŐݒ肷.

    
    * z (NArray) : 2z 
    * dx (Float) : dx0̂Ƃ dxݕƂ. dx=0̂Ƃp^ nlev QƂ,  nlev { R^[x𐶐. dx0̂Ƃ (dx.abs.to_i) {̃R^[x𐶐. 

    l
    (1) ̃\bhUDGCLBĂł.
    (2) ̃[` z ̊iq_l̍ŏlEől`FbN,  Contour.gcla Ăł. 

=end

      def sclv(zlev, index, itype, clv, hl)
	@param.sync
	
	DCL.udsclv(zlev, index, itype, clv, hl)
	nil
      end

# by DCL manual
#english
=begin
---Contour.sclv(zlev, index, itype, clv, hl)
    Specifies all of the attributes for a contour line and sets a single contour level.

    Arguments
    * zlev (Float) : The value of a contour level.
    * index (Fixnum) : The line index of a contour line.
    * itype (Fixnum) : The line type of a contour line.
    * clv (String) : The label for the contour line.
    * hl (Float) : The height of the label. (Units in VC). When 0, no labels are drawn.

    l
    (1) UDSCLV is called in this method.
    (2) Contour.gcla and Contour.gclb call this routine internally. When you wish to draw auxiliary lines, call Contour.gcla and Contour.gclb before calling this routine to add an extra contour level.  
    (3) The length of the label (CLV) must be shorter than 8.
    (4) The number of contour levels currently set can be inquired with
         Contour.levelget .
    
=end
#japanese
=begin
---Contour.sclv(zlev, index, itype, clv, hl)
    R^[Cׂ̑Ďw肵1{̃R^[xݒ肷.

    
    * zlev (Float) : R^[x̒l.
    * index (Fixnum) : R^[C̃CCfNX.
    * itype (Fixnum) : R^[C̃C^Cv.
    * clv (String) : R^[Cɂ郉x.
    * hl (Float) : x̍(PʂVWn). hl=0 ̂ƂxȂ.

    l
    (1) ̃\bhUDSCLVĂł.
    (2) Contour.gcla, Contour.gclb ͓ł̃\bhĂł. ⏕ꂽƂȂ Contour.gcla, Contour.gclb Ă񂾌ł̃\bhĂׂ, ׂR^[x1{ǉł. 
    (3) x(clv)̒ 8 ȉłȂ΂ȂȂ. 
    (4) ݐݒ肳ĂR^[x̑ 
         Contour.levelget
    ɂĎQƂł. 
    
=end

      def dclv(zlev)
	DCL.uddclv(zlev)
	nil
      end

# by DCL manual
#english
=begin
---Contour.dclv(zlev)
    Deletes a contour level.
 
    Arguments
    * zlev (Float) : The contour level to be deleted.

    Notes
    (1) UDDCLV is called in this method.
    (2) Call this routine when you wish to delete a specific contour level specified beforehand using Contour.gcla and Contour.gclb.

=end
#japanese
=begin
---Contour.dclv(zlev)
    R^[x폜. 
 
    
    * zlev (Float) : 폜R^[x̒l.

    l
    (1) ̃\bhUDDCLVĂł.
    (2) Contour.gcla, Contour.gclb ɂĊɐݒ肳ĂR^[x̃R^[x폜Ƃ̃[`Ăׂ΂悢. 

=end

      def iclv
	DCL.udiclv
	nil
      end

# by DCL manual
#english
=begin
---Contour.iclv
    Clears a contour level set using Contour.gcla, Contour.gclb and Contour.sclv.

    Arguments
    None

    Notes
    (1) UDICLV is called in this method.
    (2) This routine must be called when you wish to clear the contour levels set beforehand by Contour.gcla, Contour.gclb, and Contour.sclv.

=end
#japanese
=begin
---Contour.iclv
    Contour.gcla, Contour.gclb, Contour.sclv Őݒ肵R^[x𖳌ɂ. 

    
    Ȃ

    l
    (1) ̃\bhUDICLVĂł.
    (2) Contour.gcla, Contour.gclb, Contour.sclv ɂĊɐݒ肳ĂR^[x𖳌ɂ Contour.draw ĂтƂ, ̃[`ĂԕKv. 

=end

      def intervalget(nlev)
	DCL.rudlev(nlev)
      end

# by DCL manual
#english
=begin
---Contour.intervalget(nlev)
    Determines the interval of the contour level set by Contour.gcla, Contour.gclb, and Contour.sclv.

    Arguments
    * nlev (Fixnum) : Specifies which contour interval to check. (the NLEV th contour interval.)

    Notes
    (1) RUDLEV is called in this method.
    (2) Internally, the values of the (NLEV)th and (NLEV+1)th contour levels are checked by UDQCLV, and the difference (the (NLEV+1)th value minus the (NLEV)th value) is returned. Note that the values of the contour levels inquired by UDQCLV are not necessarily in ascending order. 
    (3) Two or more contour levels must be set. Also, NLEV must be larger than 1 and smaller than the total number of contour levels set. When this condition is not satisfied, RUDVAL gives an alert message and 0 is returned. 

=end
#japanese
=begin
---Contour.intervalget(nlev)
    DGCLA, UDGCLB, UDSCLVŐݒ肵R^[x Ԋu߂. 

    
    * nlev (Fixnum) : Ԗڂ̃R^[Ԋu𒲂ׂ邩w肷.

    l
    (1) ̃\bhRUDLEVĂł.
    (2) IɂUDQCLV ɂđ nlev Ԗڂ nlev+1 Ԗڂ R^[x̒l𒲂, ̍( nlev+1 Ԗڂ nlev Ԗڂl)ԂDUDQCLVQƂR^[x̒l͕KɂȂłƂ͌ȂƂɒӂ邱.
    (3) R^[x 2 {ȏݒ肳ĂȂ΂ȂȂ. ܂ nlev  1 ȏ, ݒ肳ĂR^[x̑ 菬Ȃ΂ȂȂD̏ȂƂ RUDVAL ͌xbZ[Wo, 0 Ԃ. 

=end

      def levelget
	nlev = DCL.udqcln
	levels = []
	for i in 1..nlev
	  status = DCL.udqclv(i)
	  levels.push(status)
	end
	levels
      end
      
# by kawanabe
#english
=begin
---Contour.levelget
    Return specified attributes of contour levels.

    Arguments
    None

    Notes
    (1) Return an obejct of [[zlev, index, itype, clv, hl], [zlev, ... ], ...].

=end
#japanese
=begin
---Contour.levelget
    ݐݒ肳ĂR^[x̑߂B

    
    Ȃ

    l
    (1) ԂĺA[[zlev, index, itype, clv, hl], [zlev, ... ], ...] Ƃ`ArrayIuWFNgłB

=end
#----------------------------------------------------------
#------------ draw method ----------------------
      def draw(data, x=nil, y=nil, temp=nil)
	if(!(y.is_a?(Hash)))
	  if(x.is_a?(Hash))
	    temp = x.dup
	    x = nil
	  elsif(Basics.typearray(x))
	    if(!Basics.typearray(y))
	      raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	    end
	  end
	else
	  raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	end
	
	nx = data.shape[0]
	ny = data.shape[1]
	
	if(Basics.typearray(x) && Basics.typearray(y))
	  DCL.uwsgxa(x, x.length)
	  DCL.uwsgya(y, y.length)
	  Basics.checkundef(x, y)
	else
	  Basics.checkundef(nx.to_f, ny.to_f)
	end
	
	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	nb = DCL.gliget("nbitspw")
	nbr = (3*((nx+2)*(ny+2)*2.0/nb + 1)).ceil
	DCL.udcntz(data, nx, nx, ny, nbr)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end
      
# by DCL manual
#japanese
=begin
---Contour.draw(data, x=nil, y=nil, temp=nil)
    2}`

    
    * data (NArray) : 2z
    * x, y (Array or NArray) : xWAyW^z
    * temp (Hash) : e|p^

    l
    (1) ̃\bhUDCNTZĂłB
    (2) iq_̊ԊusԊȕꍇAƂx, y^ĂƎIɊiq_ݒ肷B
    (3) EBhE̐ݒsĂȂꍇɂāAx, y^΂ƂɃEBhEݒsAx, y^ȂΔz data ̊e̒ƂɃEBhEݒ肷B
    (4) R^[x̒lContour.gcla, Contour.gclb ܂ Contour.sclv ɂĐݒ肳ĂȂꍇ, ̃Tu[`ŌĂ  nlev {̃R^[xݒ肷. 
          Contour.gclb(z, -(nlev)) 
        nlev  Contour ̊Ǘp^ nlev QƂlł(l12).
    (5) R^[ASY̐񂩂, iq_̒l͂ǂ̃R^[x̒lƂĂ͂ȂȂ. ̂悤Ȋiq_lƓŔʂC (̍ۃbZ[Wo͂). 
    (6) Fig ̊Ǘp^ lmiss  true Ȃ猇lȂ. (̍, QƂ錇l Fig ̊Ǘ p^ rmiss ł. ) false Ȃ猇lȂȂ.  
    (7) R^ȎOɂׂĂ̊iq_l̃`FbNȂ. 2̏ꍇR^O͂Ȃ킸̎|bZ[Wo͂. Ƀp^ lmsg  true ̂ƂOt̉}[WɂbZ[W. false ̂ƂbZ[WȂ (l true ).2̏ꍇƂ: 
          * ׂĂ̊iq_l̂Ƃ(missing field). 
          * ׂĂ̊iq_Ȓl̂Ƃ(constant field). 
    (8) p^ lmsg  true ňׂR^[x̐2{ȏ゠Ƃ, ^̊֐RUDLEV(1) Ă 1R^[x̒l(CZL1)Ƒ2R^[x̒l(CZL2)̍ (CZL2-CZL1)Ot̉}[Wɏo. ̃bZ[W, n}eϊȊOݒ肳ĂƂ, Axis Ǘgď. , ̂悤ȏꍇ, ܂W` Contour.draw ĂԂׂł. ۂ, n}eϊݒ肳ĂƂ, }[W̐^ɏ. 

=end
#english
=begin
---Contour.draw(data, x=nil, y=nil, temp=nil)
    Draws a 2-D contour graph (using a work area provided internally).  

    Arguments
    * data (NArray) : a 2-D array
    * x, y (Array or NArray) : arrays of x-coordinate and y-coordinate
    * temp (Hash) : temoporary parameters

    Notes
    (1) UDCNTZ is called in this method.
    (2) iq_̊ԊusԊȕꍇAƂx, y^ĂƎIɊiq_ݒ肷B
    (3) When the window is not specified, the window is specified from x and y if they are specified, and window is specified from the shape of data if they are not specified.
    (4) If the value of the contour level is not set by Contour.gcla, Contour.gclb, or Contour.sclv, then the following subroutine is called internally to set approximately NLEV contour levels. 
          Contour.gclb(z, -(nlev))
        NLEV is the value inquired by the internal variable 'nlev' handled by the Contour module. (The initial value is 12.).
    (5) From restrictions for algorithms for drawing a contour, the values of the grid points must not be equal to any value of the contour level. If such a grid-point value exists, a minute correction is made internally (and a message is printed to indicate that a correction had been made.)
    (6) Missing-value handling is performed when the internal variable 'lmiss' handled by the Fig module is true (The missing value inquired in this case is the internal variable 'rmiss' handled by the Fig module.) When false, missing-value handling is not performed. 
    (7) Before contouring, all grid-point values are checked. In the following 2 cases, contouring is not performed, and a message is printed. Furthermore, when the internal variable 'lmsg' is true, then a message is also printed in the bottom margin. When false, the message is not printed. (The initial value is true.) The 2 cases are:   
        * when all grid points are missing values (a missing field)  
        * when all grid points have the same value (a constant field).
    (8) When the internal variable 'lmsg' is true and there are more than 2 contours to draw, the floating-point function Contour.intervalget(1) is called to print the difference (CZL2-CZL1) between contour level one (CZL1) and contour level 2 (CZL2) in the bottom margin of the graph. These messages are written using the information handled by the Axis::Lin module when a transformation other than the map projection transformation is set. Therefore, in a case such as this, the coordinate axes should be plotted before a call is made to Contour.draw. On the other hand, when the map projection transformation is specified, then the message is printed at the center of the bottom margin.

=end
      module_function :param_init
      module_function :help_params
      module_function :set
      module_function :get
      module_function :sync
      module_function :reset
      module_function :draw
      module_function :gcla
      module_function :gclb
      module_function :sclv
      module_function :dclv
      module_function :iclv
      module_function :intervalget
      module_function :levelget
      
      param_init

#------------------- singleton methods ----------------------
  
      def @param.indexmj(index)
	DCL.udiset("indxmj", index)
	nil
      end

      def @temp_param.indexmj(index)
	DCL.udiset("indxmj", index)
	nil
      end
    
      def @backup.indexmj(index)
	DCL.udiset("indxmj", index)
	nil
      end

      def @param.indexmjget
	DCL.udiget("indxmj")
      end

      def @param.indexmn(index)
	DCL.udiset("indxmn", index)
	nil
      end

      def @temp_param.indexmn(index)
	DCL.udiset("indxmn", index)
	nil
      end

      def @backup.indexmn(index)
	DCL.udiset("indxmn", index)
	nil
      end

      def @param.indexmnget
	DCL.udiget("indxmn")
      end

      def @param.labelsize(size)
	DCL.udrset("rsizel", size)
	nil
      end

      def @temp_param.labelsize(size)
	DCL.udrset("rsizel", size)
	nil
      end

      def @backup.labelsize(size)
	DCL.udrset("rsizel", size)
	nil
      end

      def @param.labelsizeget
	DCL.udrget("rsizel")
      end

      def @param.titlesize(size)
	DCL.udrset("rsizet", size)
	nil
      end

      def @temp_param.titlesize(size)
	DCL.udrset("rsizet", size)
	nil
      end

      def @backup.titlesize(size)
	DCL.udrset("rsizet", size)
	nil
      end
      
      def @param.titlesizeget
	DCL.udrget("rsizet")
      end

#----------------------------------------------------------

      @param.keys.each{|i|
	eval "def #{i}; @param['#{i}']; end"
	eval "def #{i}=(val); set('#{i}', val); end"
      }
      
      @param.keys.each{|i|
	eval "module_function :#{i}"
	eval "module_function :#{i}="
      }
    end
  end
end
